package com.microfocus.da.sct.samples;

import com.microfocus.da.sct.annotations.SourceConfigTypeConfig;
import com.microfocus.da.sct.annotations.SourceConfigTypeParam;
import com.microfocus.da.sct.dtos.VersionInfo;
import com.microfocus.da.sct.interfaces.CommonIntegrator;
import com.microfocus.da.sct.interfaces.ComponentInfo;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;

import java.io.File;
import java.util.*;

/**
 * Custom File System Versioned SCT
 *
 */
@SourceConfigTypeConfig(name="Sample File System (Versioned)", versionNumber = "1.0", description = "Sample Source Config type for connecting File System (Versioned) repositories")
public class FileSystemVersioned implements CommonIntegrator {

    @SourceConfigTypeParam(displayName = "Base Path", description = "Base path for artifact storage", required = true)
    private String basePath;

    private ComponentInfo componentInfo;

    @Override
    public Collection<VersionInfo> getNextVersionInfo(Properties properties) throws Exception {

        File toCreate = getNewVersionDir();

        if(toCreate == null)
            return null;

        // This is when we can really create a version now.
        VersionInfo toSend = new VersionInfo(toCreate.getName());
        toSend.setVersionProps(properties);

        return Collections.singletonList(toSend);
    }

    @Override
    public void downloadVersionContent(VersionInfo versionInfo, File processingDirectoryLocation) throws Exception {

        File latestFile = getBaseDir();

        if(latestFile == null)
            return;

        FileUtils.copyDirectory(new File(latestFile, versionInfo.getVersionName()), processingDirectoryLocation, true);
    }

    @Override
    public Collection<String> getAlerts(Locale locale) throws Exception {
        List<String> result = new ArrayList<String>();

        File baseDir = getBaseDir();

        if(baseDir == null || !baseDir.exists())
            result.add("The base filesystem path for this component does not exist.");
        else if (!baseDir.isDirectory())
            result.add("The base filesystem path doesn't point to a directory.");

        return result;
    }

    /**
     * Determines what could be the next DIrectory which needs to be versioned
     * to the system.
     *
     * @return
     * @throws Exception
     */
    private File getNewVersionDir() throws Exception  {

        Collection<String> alerts = getAlerts(null);

        // There are base path issues, we cannot create new version
        if(alerts != null && alerts.size() > 0)
            return null;

        for (File versionDir : getBaseDir().listFiles()) {

            if(!versionDir.isDirectory())
                continue;

            if(componentInfo.getVersionByName(versionDir.getName()) != null)
                continue;

             return versionDir;
        }

        return null;
    }

    public File getBaseDir() {

        if(StringUtils.isEmpty(basePath))
            return null;

        return new File(basePath);
    }
}